// This javascript is executed only on chrome://settings and http://127.0.0.1:8080 (JMUConfig). The code here can be used to customize
// chrome settings, add, hide and modify entries

// Flag to let jmuconfig know we are on local chromium
sessionStorage.setItem("localChromium", "true");

function getSettingsSection(sectName) {

 var mainPage = document.getElementsByTagName("settings-ui")[0].shadowRoot.querySelector("#main").shadowRoot;
 return mainPage.querySelector("settings-basic-page").shadowRoot.querySelector("#" + sectName);
}

function handleToggle(id, setting_name) {
   var container = getSettingsSection(id);
   if (container) {
       var toggle = container.shadowRoot.querySelector("cr-toggle");
       if (toggle) {
          if (settings[setting_name])
             toggle.setAttribute("checked", true);
          else
             toggle.removeAttribute("checked");

          container.onpointerup = function(e) {
             var settings = {};
             settings[setting_name] = !toggle.hasAttribute("checked");
             chrome.extension.sendMessage("", { cmd: "setSettings", settings: settings });
             if ( toggle.hasAttribute("checked") )
                toggle.removeAttribute("checked");
             else
                toggle.setAttribute("checked", true);
             e.stopPropagation();
          };
       }
   }
}

var settings = {};
var flags = {};

document.addEventListener("DOMContentLoaded", function(e){

   chrome.runtime.sendMessage("", { cmd: "getInfo" }, function(info) {
      if ( info ) {
         if ( info.settings )
            settings = info.settings;

         if ( info.flags )
            flags = info.flags;
      }

      if ( document.location.href.startsWith('http://127.0.0.1:8080') ) {

         if (info.tabIndex != 1)
             return;

         // Elements are recreated every time the module is opened.
         // Event listeners on DOM elements needs to registered every time.
         document.addEventListener("DOMNodeInserted", function(e) {
             if (e.target.id == "ext_certificate") {

                 certificateBtn = document.getElementById("ext_certificate_manager");
                 if (certificateBtn)
                     certificateBtn.addEventListener("pointerup", function() {
                         chrome.extension.sendMessage("", { cmd: "certificates" });
                     });
             }

	 }, false);

         var jmuconfigExit = null;

         // Hook to logout() action. We need a waiting loop as elements are
         // dynamically generated by angualar
         var timer = window.setInterval(function(){

            // This should match JMUConfig elements calling logout()
            jmuconfigExit = document.querySelectorAll('[ng-click="logout()"]');
            if (jmuconfigExit && jmuconfigExit.length > 0 ){
               window.clearInterval(timer);
               jmuconfigExit.forEach(function(exitAction) {
                  exitAction.addEventListener("click", function() {
                     chrome.extension.sendMessage("", { cmd: "back" });
                  });
               });
            }
         }, 1000);

         return;
      }
   });

   // "Back" button on the left settings menu
   var backEntry = document.getElementsByTagName("settings-ui")[0].shadowRoot.querySelector("cr-toolbar").shadowRoot.querySelector("#ext_back");
   if (backEntry) {
      backEntry.addEventListener("pointerup", function() {
         chrome.extension.sendMessage("", { cmd: "closeCertificates" });
      });
   }
});

// Disable context menu
window.addEventListener("contextmenu", function(e) { e.preventDefault(); })
