// Loaded on options.html

var extensionInfo = {};

var exitBtn = null;
var homepageBtn = null;
var showLoginRetryAfterCDown = false;

var jmuconfigLoadLoginPageRetries = { max: 10, delay: 1000 };
var jmuconfigCheckRecoveryRetries = { max: 4, delay: 3000 };

chrome.runtime.onMessage.addListener(function(msg, sender, sendResponse) {
   if ( msg.settings ) {
      extensionInfo.settings = msg.settings;
      updateElementsVisibility();
   }
});

document.addEventListener("DOMContentLoaded", function() {

   loadBackendLoginPage();

   // reload login page when the "option" tab become active
   chrome.tabs.onActivated.addListener(function(activeInfo) {
      if (activeInfo.tabId == 2)
         loadBackendLoginPage();
   });
});

function setupPageElements() {

   // Get Settings
   chrome.runtime.sendMessage("", { cmd: "getInfo" }, function(info) {

      extensionInfo = info;

      // create menu items
      createMainMenuItem("menu_exit", "Exit", true);
      createMainMenuItem("menu_homepage", "Load Homepage", true);
      createMainMenuItem("menu_clearCache", "Clear cache", true);
      createMainMenuItem("menu_keep-alive", "Keep alive", true);
      createMainMenuItem("menu_refresh-page", "Refresh page", true);

      // setup hidden elements
      document.getElementById("back").classList.remove("hidden");
      document.getElementById("creditsBox").classList.remove("hidden");

      const backButton = document.getElementById("back");
      if (backButton) {
         if ( info.tabIndex && info.tabIndex != 0 ) {
            backButton.addEventListener( "click", function(e) {
               chrome.tabs.getCurrent(function(tab) {
                  chrome.extension.sendMessage({ cmd: "back" });
               });
            });
         } else {
            backButton.classList.add("disabled");
         }
      }

      exitBtn = document.getElementById("menu_exit");
      exitBtn.classList.remove("hidden");
      exitBtn.addEventListener( "click", function(e) {
         chrome.tabs.getCurrent(function(tab) {
            chrome.extension.sendMessage({ cmd: "exitChrome" });
         });
      });

      homepageBtn = document.getElementById("menu_homepage");
      homepageBtn.classList.remove("hidden");
      homepageBtn.addEventListener( "click", function(e) {
         chrome.tabs.getCurrent(function(tab) {
            chrome.extension.sendMessage({ cmd: "back", url: extensionInfo.settings.homepage });
         });
      });

      creditsLink = document.getElementById("credits_link");
      creditsLink.addEventListener( "click", function(e) {
          chrome.extension.sendMessage({ cmd: "credits" });
      });

      var clearCacheBtn = document.getElementById("menu_clearCache");
      clearCacheBtn.classList.remove("hidden");
      clearCacheBtn.addEventListener( "click", function(e) {
         chrome.extension.sendMessage({ cmd: "clearCache" });
      });

      const recoveryLink = document.getElementById("recovery_link");
      if (recoveryLink) {
         recoveryLink.addEventListener( "click", function(e) {
            chrome.extension.sendMessage({ cmd: "recovery" });
         });
      }

      const userLoggedIn = (document.getElementById("loggedIn").innerHTML == "true");
      if (userLoggedIn) {
         const submitLogout = document.getElementById("logout");
         submitLogout.addEventListener("click", function(e) {
            e.preventDefault();
            sessionLogout();
         });

         const showSessionInfo = (document.getElementById("sessionInfo").innerHTML == "true");
         if (showSessionInfo) {
            const submitKAlive = document.getElementById("menu_keep-alive");
            submitKAlive.classList.remove("hidden");
            submitKAlive.addEventListener("click", function(e) {
               e.preventDefault();
               sessionKeepAlive(true);
            });

            const submitRefreshPage = document.getElementById("menu_refresh-page");
            submitRefreshPage.classList.remove("hidden");
            submitRefreshPage.addEventListener("click", function(e) {
               e.preventDefault();
               location.reload();
            });
         }
      }
      else {
         const expiredReasonMsg = document.getElementById("expiredReason").innerHTML;
         if (expiredReasonMsg !== "" && expiredReasonMsg !== "none")
            setHeaderMessage(expiredReasonMsg, true);
      }

      updateElementsVisibility();

//      checkRecoverySupport();
   });

   const submitLogin = document.getElementById("loginForm");
   if (submitLogin) {
      submitLogin.addEventListener( "submit", function(e) {
         e.preventDefault();
         sessionLogin();
         return false;
      });
   }

}

function createMainMenuItem(id, label, hidden) {

   hidden = (typeof hidden === "undefined") ? false : hidden;

   let elem = document.getElementById(id);
   if (elem)
      return elem;

   // create new elem
   li_elem = document.createElement("li");
   const a_elem = document.createElement("a");
   a_elem.appendChild(document.createTextNode(label));
   li_elem.appendChild(a_elem);
   li_elem.setAttribute("id", id);
   if (hidden)
      li_elem.setAttribute("class", "hidden");

   // append to mainMenu
   const parent = document.getElementById("mainMenu");
   if (parent)
      parent.appendChild(li_elem);
   return li_elem;

}

function updateElementsVisibility() {

   exitBtn = document.getElementById("menu_exit");
   if ( extensionInfo.flags.enableExitButton )
      exitBtn.classList.remove("hidden");
   else
      exitBtn.classList.add("hidden");

   if ( extensionInfo.settings.homepage && extensionInfo.settings.homepage != "" )
      homepageBtn.classList.remove("hidden");
   else
      homepageBtn.classList.add("hidden");
}

function setHeaderMessage(msg, err) {

    var msgHeader = document.getElementById("msgBox");

    if (!msg) {
      msgHeader.style.visibility = "hidden";
      return;
    }

    msgHeader.innerHTML = msg;
    if (err)
        msgHeader.style.background = "#961114";
    else
        msgHeader.style.background = "#008518"

    msgHeader.style.visibility = "visible";
}

function getCookie(cookies, name) {

   for (var i = 0; i < cookies.length; ++i) {
      if (cookies[i].name == name)
         return cookies[i].value;
   }

   return "";
}

function sessionLogin() {

   const passwdElem = document.getElementById("password");
   const password = passwdElem.value;
   const username = document.getElementById("username").value;

   passwdElem.value = "";

   if(!username)
      return;

   chrome.cookies.getAll({}, function(cookies) {

      const xmlHttp = new XMLHttpRequest();
      xmlHttp.open("POST", "http://127.0.0.1:8080/restsession/api/v1/authentication/login", true);
      xmlHttp.setRequestHeader("Content-Type", "application/json");
      xmlHttp.setRequestHeader("X-XSRF-TOKEN", getCookie(cookies, "XSRF-TOKEN"));
      xmlHttp.onreadystatechange = function() {
         if (xmlHttp.readyState === 4) {
            if (xmlHttp.status === 200) {
               // auth success
               chrome.extension.sendMessage({ cmd: "login" });
               return;
            }
            // auth failure, give feedback to user
            var msg = "Authorisation attempt failed";
            var data = JSON.parse(xmlHttp.responseText);
            if (data && data.message)
               msg = data.message;
            setHeaderMessage(msg, true);
         }
      }
      xmlHttp.send(JSON.stringify({ username: username, password: password }));

   });
}

function sessionLogout() {
   chrome.cookies.getAll({}, function(cookies) {

      const xmlHttp = new XMLHttpRequest();
      xmlHttp.open("POST", "http://127.0.0.1:8080/restsession/api/v1/authentication/logout", true);
      xmlHttp.setRequestHeader("X-XSRF-TOKEN", getCookie(cookies, "XSRF-TOKEN"));
      xmlHttp.onreadystatechange = function() {
         if (xmlHttp.readyState === 4 && xmlHttp.status === 200) {
            setupPageElements();
            return;
         }
      };
      xmlHttp.send();

   });
 }

 function sessionKeepAlive(reload) {
   reload = (typeof reload === "undefined") ? false : reload;

   chrome.cookies.getAll({}, function(cookies) {

      const xmlHttp = new XMLHttpRequest();
      xmlHttp.open("POST", "http://127.0.0.1:8080/restsession/api/v1/authentication/keep-alive", true);
      xmlHttp.setRequestHeader("X-XSRF-TOKEN", getCookie(cookies, "XSRF-TOKEN"));
      xmlHttp.onreadystatechange = function() {
         if (xmlHttp.readyState !== 4)
            return;
         if (reload)
            window.location.reload();
      };
      xmlHttp.send();

   });
 }

function loadBackendLoginPage(retryNum) {

   retryNum = (typeof retryNum === "undefined") ? jmuconfigLoadLoginPageRetries.max : retryNum;

   var xmlHttp = new XMLHttpRequest();

   xmlHttp.onreadystatechange = function() {

      // If we get unauthorized we need to fallback to the basic auth implementation
      if (xmlHttp.status == 401) {
         chrome.extension.sendMessage({ cmd: "openOptionsBasicAuth" });
         return;
      }

      if ( xmlHttp.readyState != 4 )
         return;

      if ( xmlHttp.status == 200 ) {

         var html = xmlHttp.response;
         document.getElementById("backendLoginPage").innerHTML = html;

	      setupPageElements();

      } else if ( retryNum > 0 ) {
         setTimeout(loadBackendLoginPage, jmuconfigLoadLoginPageRetries.delay, retryNum-1);
      } else {
         document.getElementById("backendLoginPage").innerHTML = "<b>Error loading login page</b>";
      }
   }

   xmlHttp.open("GET", "http://p:p@127.0.0.1:8080/login", true);
   xmlHttp.send(null);
}

function checkRecoverySupport(retryNum) {

   retryNum = (typeof retryNum === "undefined") ? jmuconfigCheckRecoveryRetries.max : retryNum;

   var xmlHttpRecoverySupport = new XMLHttpRequest();

   xmlHttpRecoverySupport.onreadystatechange = function() {
      if ( xmlHttpRecoverySupport.readyState != 4 )
         return;

      if ( xmlHttpRecoverySupport.status == 200 ) {

	      var result = JSON.parse(xmlHttpRecoverySupport.responseText);

         if ( result.status >= 0 ) {
            recoveryLink = document.getElementById("forgotMsgDiv");
            recoveryLink.classList.remove("hidden");
         }

      } else if ( retryNum > 0 ) {
         setTimeout(checkRecoverySupport, jmuconfigCheckRecoveryRetries.delay, retryNum-1);
      }
   }

   xmlHttpRecoverySupport.open("GET", "http://none:none@127.0.0.1:8080/devicerestore/getStatus", true);
   xmlHttpRecoverySupport.send(null);
}
