#!/bin/bash

cd $(dirname $0)
SCRIPTDIR=$(pwd)
echo $SCRIPTDIR

export PATH="$PATH:$SCRIPTDIR/bin"

function sync_pw_db {

	# Chromium DB file and Exor copy
	CHROMIUM_PWD_DB_FILE="$DATA_DIR/Default/Login\ Data"
	CHROMIUM_PWD_DB_COPY_FILE="$DATA_DIR/Default/exor.db"

	# Import modified password DB (if any) - then remove
	if [ -f $CHROMIUM_PWD_DB_COPY_FILE ]; then
		eval /usr/bin/chromium-db $CHROMIUM_PWD_DB_COPY_FILE $CHROMIUM_PWD_DB_FILE
		rm -rf $CHROMIUM_PWD_DB_COPY_FILE
	fi

	# Create working copy for password DB
	CHROMIUM_TMP_EXPORT_FILE=$(mktemp /tmp/logins-sql.XXXXXX)

	exportsql='".output '$CHROMIUM_TMP_EXPORT_FILE'" ".dump logins " ".quit"'
	importsql=$(cat << __EOF__
".read $CHROMIUM_TMP_EXPORT_FILE"
"ALTER TABLE logins ADD ins INTEGER DEFAULT 0;"
"ALTER TABLE logins ADD upd INTEGER DEFAULT 0;"
"ALTER TABLE logins ADD del INTEGER DEFAULT 0;"
"UPDATE logins SET ins=1, del=1 WHERE blacklisted_by_user == 1"
".quit"
__EOF__
)

	eval sqlite3 $CHROMIUM_PWD_DB_FILE $exportsql
	eval sqlite3 $CHROMIUM_PWD_DB_COPY_FILE $importsql
	[ $? -ne 0 ] && cp $SCRIPTDIR/exor.db $CHROMIUM_PWD_DB_COPY_FILE

	rm $CHROMIUM_TMP_EXPORT_FILE
}


#check browser.done file
if [ ! -f browser.done -a -f browser.ini ]; then
    echo -e "var browser_settings =" `sed -f ini2json.sed browser.ini` > ext/browser_settings.js
    DEF_USER_AGENT="$( cat browser.ini | grep customUserAgent | sed 's/^;.*//' | cut -d'=' -f2 | tail -n1 )"
    touch browser.done
else
   echo "var browser_settings = {}" > "ext/browser_settings.js"
fi

[ -e /etc/profile ] && . /etc/profile

if [ -d $HOME/.dbus/session-bus ]; then
	. $HOME/.dbus/session-bus/*
	export DBUS_SESSION_BUS_ADDRESS
fi

DATA_DIR="$SCRIPTDIR/.chromium/"
CACHE_DIR="$DATA_DIR/Default/Cache"
EXTENSION_DIR="$SCRIPTDIR/ext/"
EXTENSION_CACHE_DIR="$DATA_DIR/Default/Local Extension Settings/pppblgnjgngbhkjehblmbfeklkbfamdm/"
EXTENSION_CACHE_LOCK="$EXTENSION_CACHE_DIR/LOCK"
EXTENSION_CMD_FILE="$EXTENSION_DIR/cmd_line.js"
CHROME_WINDOW_FLAG="/run/chrome-window"
CACHE_SIZE_MB=60
WAYLAND="$( test -e /tmp/runtime-root/wayland-0.lock && echo 1 )"

TOTAL_RAM_KB="$( cat /proc/meminfo | grep MemTotal | awk '{print $2}' )"
[ "$TOTAL_RAM_KB" -lt "250000" ] && CACHE_SIZE_MB=30

if !( df -h $CACHE_DIR 2>/dev/null | sed -n 2p | grep -q ^tmpfs ); then
	mkdir -p $CACHE_DIR
	if [ -x /usr/bin/mount_tmpfs ]; then
		mount_tmpfs $CACHE_DIR $(( $CACHE_SIZE_MB +3 ))
	else
		mount -t tmpfs -o size=$(( $CACHE_SIZE_MB +3 ))M tmpfs $CACHE_DIR
	fi
fi

CPU="$( cat /proc/cpuinfo | grep Hardware )"
if [ -n "$WAYLAND" ]; then
	GPU_FLAGS="--use-gl=egl --ozone-platform=wayland --in-process-gpu"
elif ( echo $CPU | grep -q "Freescale i.MX6" ) && ( lsmod | grep -q galcore ); then
	GPU_FLAGS="--use-gl=egl --enable-zero-copy --enable-accelerated-2d-canvas \
                --enable-gpu-compositing --enable-threaded-compositing  --gpu-no-context-lost \
                --enable-gpu-memory-buffer-video-frames --enable-native-gpu-memory-buffers"
else
	GPU_FLAGS="--disable-gpu"
fi

CMD_LINE=""

[ -e $CHROME_WINDOW_FLAG ] && exit 0
touch $CHROME_WINDOW_FLAG


declare -A extDB
if [ -d "$EXTENSION_CACHE_DIR" ]; then
	while read line; do
		variable="$( echo $line | cut -d":" -f1 | xargs )"
		value="$( echo $line | cut -d":" -f2- | xargs | sed 's:^"\(.*\)"$:\1:' )"
		extDB["$variable"]="$value"
	done < <( LD_LIBRARY_PATH=$SCRIPTDIR/lib leveldbdump "$EXTENSION_CACHE_DIR" 2>/dev/null )
fi

disabledFeatures="TranslateUI"
if [ "${extDB[overscrollHistoryNavigation]}" = "false" ]; then
	disabledFeatures="$disabledFeatures,OverscrollHistoryNavigation"
fi

CHROME_ARGS="--no-sandbox --extensions-on-chrome-urls --disable-session-crashed-bubble --disable-features=$disabledFeatures --disable-print-preview \
        --disable-infobars --no-first-run --kiosk --disable-component-update --disable-sync \
        --user-data-dir=$DATA_DIR --disk-cache-size=$(( $CACHE_SIZE_MB *1024 *1024 )) $GPU_FLAGS"

[ -n "$WAYLAND" ] && CHROME_ARGS="$CHROME_ARGS --enable-wayland-ime"

# Uncomment to enable verbose log
# CHROME_ARGS="$CHROME_ARGS --enable-logging=stderr --v=1"

# Uncomment to enable remote debugging on port 9222
# CHROME_ARGS="$CHROME_ARGS --remote-debugging-port=9222"

# Uncomment and set <IP> and <PORT> to enable proxy
# CHROME_ARGS="$CHROME_ARGS --proxy-bypass-list=127.0.0.1 --proxy-server=<IP>:<PORT>"

if [ -z "$WAYLAND" ]; then
	# Non multitouch devices are not recognized as touchscreens. We need to force chromium to believe they are
	titscdev=$( /usr/bin/xinput | grep ti-tsc | sed "s:.*id=\([0-9]*\).*:\1:" )
	[ -n "$titscdev" ] && CHROME_ARGS="$CHROME_ARGS --touch-devices=$titscdev --touch-events"
	tsc200xdev=$( /usr/bin/xinput | grep 'TSC200\(4\|X\)' | sed "s:.*id=\([0-9]*\).*:\1:" )
	[ -n "$tsc200xdev" ] && CHROME_ARGS="$CHROME_ARGS --touch-devices=$tsc200xdev --touch-events"
fi

CHROME_ARGS_BG="$CHROME_ARGS --load-extension=$EXTENSION_DIR --no-startup-window"

[ -n "${extDB[customUserAgent]}" ] && USER_AGENT="--user-agent=${extDB[customUserAgent]}"
[ -n "$DEF_USER_AGENT" ] && USER_AGENT="--user-agent=$DEF_USER_AGENT"

if [ -e $DATA_DIR/Default/Preferences ]; then

	# Chromium crash detection triggers every time the panel is powered off without properly closing the applciation. We have
	# to clear out this state otherwise Chromium will refuse to load saved settings, probably considering safer this way.
	sed -i'' 's/"exit_type":"Crashed"/"exit_type":"Normal"/' $DATA_DIR/Default/Preferences

	if [ "${extDB[clearCertificateDecisions]}" = "true" ]; then

		LD_LIBRARY_PATH=$SCRIPTDIR/lib jq -c '.profile.content_settings.exceptions.ssl_cert_decisions = {}' $DATA_DIR/Default/Preferences > $DATA_DIR/Default/Preferences.new
		if [ $? -eq 0 ]; then
			mv $DATA_DIR/Default/Preferences.new $DATA_DIR/Default/Preferences
			sync
		else
			echo "WARNING: failed to clear certificate decisions"
		fi
	fi
fi

sync_pw_db

sleep 2

( pidof chromium-bin &>/dev/null ) && daemonRunning=1

if [ ! -n "$daemonRunning" ]; then
	rm -rf $DATA_DIR/Singleton*
	rm -f "$EXTENSION_CACHE_LOCK"
fi

ulimit -c unlimited

(
	export GTK_IM_MODULE_FILE=

	# Wait for daemon socket and extension cache dir to be created
	while [ ! -e $DATA_DIR/SingletonSocket -o ! -e "$EXTENSION_CACHE_LOCK" ]; do
		sleep 1
	done;

	dbus-send --print-reply --system --dest=com.exor.JMLauncher '/' com.exor.JMLauncher.appLoaded string:"Chromium"

	# Open a browser window
	LD_LIBRARY_PATH=$SCRIPTDIR/lib \
		./chromium-bin $CHROME_ARGS $@

	rm -f $CHROME_WINDOW_FLAG
) &

if [ -n "$daemonRunning" ]; then
	echo Daemon already running
	sleep 10
	exit 0
fi

# Pass command line to extension
[ -e settings ] && CMD_LINE="$( cat settings )"
CMD_LINE="$CMD_LINE $@"
echo 'var cmd_line = "'$CMD_LINE'";' > $EXTENSION_CMD_FILE
echo 'var inst_dir = "'$SCRIPTDIR'";' >> $EXTENSION_CMD_FILE

# Run the daemon fist to make sure the extension is loaded
LD_LIBRARY_PATH=$SCRIPTDIR/lib \
	./chromium-bin $CHROME_ARGS_BG "$USER_AGENT"

if [ $? -eq "137" -a ! -e $DATA_DIR/SingletonSocket ]; then
	sleep 2
	LD_LIBRARY_PATH=$SCRIPTDIR/lib \
		./chromium-bin $CHROME_ARGS_BG "$USER_AGENT"
fi

rm -f $CHROME_WINDOW_FLAG

if ls core* &> /dev/null ; then

	xsplash --no-taptap --no-progress-bar &
	sleep 2

	psplash-write "MSG The browser exited unexpectedly
Please wait..."

	# Keep a maximum of 5 core dump reports
	if [ -d crash_dumps -a "$( ls crash_dumps 2>/dev/null | wc -l )" -gt "4" ]; then
		rm -rf "crash_dumps/$( ls crash_dumps | head -n1 )"
	fi

	CRASHLOGNAME="$(date +%Y%m%d%H%M%S)-crashlog"
	mkdir -p crash_dumps/${CRASHLOGNAME}
	mv core crash_dumps/${CRASHLOGNAME}
	cp ../package.info crash_dumps/${CRASHLOGNAME}
	cp /boot/version crash_dumps/${CRASHLOGNAME}
	cp -Lr /var/log crash_dumps/${CRASHLOGNAME}
	sync

	tar czvf crash_dumps/${CRASHLOGNAME}.tar.gz -C crash_dumps ${CRASHLOGNAME} &>/dev/null
	rm -r crash_dumps/${CRASHLOGNAME}
	sync

	psplash-write "QUIT"
fi
