jlog('Setting up Chromium Controller');

app.controller('jmuconfig.chromium.Controller',
[ '$scope', '$rootScope', '$window', '$http', '$interval', '$filter', 'ngDialog', 'defines', 'utils', 'auth', 'poller', 'dialog', '$timeout', '$translate', 'sys',
function ($scope, $rootScope, $window, $http, $interval, $filter, ngDialog, defines, utils, auth, poller, dialog, $timeout, $translate, sys)
{
  jlog('Initialising Chromium Controller');

  var chromiumUrl = defines.restApiUrl + '/app_chromium';
  var credentialsUrl = defines.restApiUrl + "/system/chromium/credentials";
  var window = angular.element($window);
  var fileInput = document.createElement('input');

  $scope.localFileBrowserUrl = defines.restApiUrl + '/x11/filebrowser'

  if (sessionStorage.getItem("localChromium") === "true" )
     $scope.isLocalChromium = true;
  else
     $scope.isLocalChromium = false;

  fileInput.type = 'file';
  fileInput.accept = '.csv';
  fileInput.onchange = function (e) {
      var file = e.target.files[0];

      var reader = new FileReader();
      reader.readAsText(file, 'UTF-8');

      reader.onload = function (readerEvent) {
          jlog("Sending csv data");

          $rootScope.loading = true;

          var contentCsv = readerEvent.target.result;
          fileInput.value = null;

          csvExport({data: contentCsv});
      }

      reader.onerror = function (err) {
          fileInput.value = null;
          jlog("FileReader error: " + err);
          dialog.error($translate.instant("_STATE_ERROR"), $translate.instant("_PASSWORD_IMPORT_ERROR"));
      }
  }

  $rootScope.loading = true;
  $scope.formState = { open: false };
  $scope.credentials = [];
  $scope.credentialsSupported = false;
  $scope.csvUrl = restApiUrl + "/system/chromium/csv";
  $scope.visibleCredentials = {};

  utils.init();

  $scope.scrollTop = function() {
    $window.scrollTo(0,0);
  };

  $scope.showAlert = function(msg, warn, confirm, cb)
  {
    $rootScope.alert = { message: msg, warning: warn, confirm: confirm };

    $scope.dialog = ngDialog.openConfirm({
      template: 'alertDialog',
      className: 'ngdialog-theme-default',
      showClose: false
    }).then(function (value) {
    }, function (reason) {
       if (cb)
          cb(reason);
    });
  };

  $scope.closeDialog = function()
  {
      if (!$scope.dialog)
          return;
      ngDialog.close($scope.dialog.id);
      $scope.dialog = null;
  };

  function pollPostCb()
  {
    if ( $scope.formState.open || ! $rootScope.global.apps || ! $rootScope.global.apps.chromium || ! $rootScope.global.apps.chromium.settings )
        return;

    $rootScope.loading = false;

    $scope.chromium = $rootScope.global.apps.chromium.settings;

    if(!$scope.chromium.customUserAgent)
        $scope.chromium.customUserAgent = "";
    $scope.chromium.enableCustomUserAgent = $scope.chromium.customUserAgent != "";

    $scope.chromium.enableFallbackPage = $scope.chromium.fallbackPage && $scope.chromium.fallbackPage != "";
    $scope.chromium.holdOptionsTimer = $scope.chromium.holdOptionsTimerMilliseconds/1000;

    if ( $scope.chromium.prefs && $scope.chromium.prefs.services ) {
       $scope.chromium.allowDownload = $scope.chromium.prefs.services.downloadRestrictions != 3;
    }

    $scope.tmpChromium = angular.copy($scope.chromium);
  };

  $scope.saveData = function($event) {

    $rootScope.loading = true;
    var rebootRequired = false;

    if (!$scope.tmpChromium.enableCustomUserAgent)
        $scope.tmpChromium.customUserAgent = "";

    if (!$scope.tmpChromium.enableFallbackPage)
        $scope.tmpChromium.fallbackPage = "";

    if (!$scope.tmpChromium.homepage)
        $scope.tmpChromium.homepage = "";

    if ( $scope.tmpChromium.prefs && $scope.tmpChromium.prefs.services ) {
        $scope.tmpChromium.prefs.services.downloadRestrictions = $scope.tmpChromium.allowDownload ? 0 : 3;
    }

    if ($scope.tmpChromium.onStartup == "homepage" && $scope.tmpChromium.homepage == "") {
        $rootScope.loading = false;
        $scope.showAlert($filter('translate')('_NO_HOMEPAGE_ERROR'));
        return;
    }

    if ($scope.tmpChromium.overscrollHistoryNavigation != $scope.chromium.overscrollHistoryNavigation ||
	$scope.tmpChromium.customUserAgent != $scope.chromium.customUserAgent ) {
        rebootRequired = true;
    }

    $scope.chromium = angular.copy($scope.tmpChromium);

    // make sure stale data is not polled in the meantime
    poller.skip();

    $http.post(chromiumUrl, $scope.tmpChromium)
        .success(function(data, status, headers, config) {
          $rootScope.loading = false;
          $scope.showAlert($filter('translate')('_SUCCESS'), rebootRequired ? $filter('translate')('_SUCCESS_REBOOT_REQUIRED') : null, rebootRequired, function(reason) {
              if ( rebootRequired && reason == "ok" )
                sys.restart();

          });
        })
        .error(function(data, status, headers, config) {
            if (data && data.error) {
                $scope.showAlert($filter('translate')('_CHROMIUM_SAVE_ERROR') + ': ' + data.error);
                $scope.cancelChromiumForm();
            }
            else {
                $scope.showAlert($filter('translate')('_JMU_SERVICE_ERROR'));
            }
            poller.unskip();
            $rootScope.loading = false;
        });
  };

  $scope.cancelChromiumForm = function($event) {

    jlog('Cancelled Chromium Setting ');
    $scope.tmpChromium = angular.copy($scope.chromium);
  };

  $scope.clearCertificateWarnings = function() {
    $scope.showAlert($filter('translate')('_CERTIFICATE_CLEAR_CONFIRM'), null, true, function(reason) {

      if ( reason != "ok" )
        return;

      $http.post(chromiumUrl, { "clearCertificateDecisions": true })
          .success(function(data, status, headers, config) {

            $scope.showAlert($filter('translate')('_SUCCESS'), $filter('translate')('_SUCCESS_REBOOT_REQUIRED'), true, function(reason) {
              if ( reason == "ok" )
                sys.restart();
            })
          })
          .error(function(data, status, headers, config) {

              if (data && data.error) {
                  $scope.showAlert($filter('translate')('_CHROMIUM_SAVE_ERROR') + ': ' + data.error);
              }
              else {
                  $scope.showAlert($filter('translate')('_JMU_SERVICE_ERROR'));
              }
          });
    });
  }

  $scope.toggleCredentialVisiblity = function (id) {
      if (id in $scope.visibleCredentials) {
          delete $scope.visibleCredentials[id];
      } else {
          $scope.visibleCredentials[id] = true;
      }
  };

  $scope.deleteCredential = function (credential) {
      dialog.confirm(
          $translate.instant("_CONFIRM"),
          $translate.instant("_DELETE_CREDENTIAL_CONFIRM", { url: credential.url} ),
          function (res) {
              if (res === "ok") {
                  jlog("Deleting credential with id " + credential.id);

                  $rootScope.loading = true;

                  $http
                      .delete(credentialsUrl + "/" + credential.id)
                      .success(function () {
                          loadData();
                          $rootScope.loading = false;
                      })
                      .error(function (error) {
                          jlog("Deleting credential error: " + error);
                          $rootScope.loading = false;
                          dialog.error(
                              $translate.instant("_STATE_ERROR"),
                              $translate.instant("_DELETE_CREDENTIAL_ERROR")
                          );
                      });
              }
          });
  };

  $scope.getLocalBrowserState = function(onSuccess, onSuccessArgs)
  {
    $http.get($scope.localFileBrowserUrl)
      .success(function (data, status, headers, config) {
        if (data.state === 'idle')
        {
          jlog('Browser idle - aborting');
        }
        else if (data.state === 'busy')
        {
          jlog('Browser busy - waiting');
          $timeout(function() {
              $scope.getLocalBrowserState(onSuccess, onSuccessArgs);
          }, 1000);
        }
        else if (data.state === 'file_saved' || data.state === 'file_opened')
        {
          jlog('File: ' + data.path);

          $scope.localBrowserPath = data.path;

          if (onSuccess)
            onSuccess(onSuccessArgs);
        }
      })
      .error(function (data, status, headers, config) {
        $scope.showAlert($filter('translate')('_JMU_SERVICE_ERROR') + ': ' + status);
      });
  };

  $scope.onExportCSV = function(local)
  {
    jlog('onExportCSV()');

    if (!local)
    {
      // remote get: fool browser into thinking a download link was clicked
      var link = document.createElement('a');
      link.href = $scope.csvUrl;
      link.click();
    }
    else
    {
      // local get: plain download
      $http.get($scope.csvUrl, {
          params : {
            cmd : 'local',
            filename : $scope.localBrowserPath
          }
        })
        .success(function (data, status, headers, config) {
          $scope.showAlert($filter('translate')('_SUCCESS'));
        })
        .error(function (data, status, headers, config) {
          $scope.showAlert($filter('translate')('_FATAL_ERROR') + ': ' + status);
        });
    }
  };

  $scope.onImportCSV = function(local)
  {
    jlog('onImportCSV()');

    if (!local)
      fileInput.click();
    else
      csvExport({filename: $scope.localBrowserPath});
  };

  $scope.importFile = function () {
      jlog("CSV file import");

      var local = $rootScope.clientIsLocal;

      if (local)  // local: delayed get (file browser target selection first)
      {
          $http.post($scope.localFileBrowserUrl, {
              action : 'get_open_file_name'
          })
          .success(function (data, status, headers, config) {
              $timeout(function() {
                  $scope.getLocalBrowserState($scope.onImportCSV, local);
              }, 1000);
          })
          .error(function (data, status, headers, config) {
              if (data && data.error)
                  jlog('[error] ' + data.error);
          });
      } else {
          $scope.onImportCSV(local);
      }
  }

  $scope.exportFile = function () {
      jlog("CSV file save");

      var local = $rootScope.clientIsLocal;

      if (local)  // local: delayed get (file browser target selection first)
      {
          $http.post($scope.localFileBrowserUrl, {
              action : 'get_save_file_name',
              fileType : 'export.csv'
          })
          .success(function (data, status, headers, config) {
              $timeout(function() {
                  $scope.getLocalBrowserState($scope.onExportCSV, local);
              }, 1000);
          })
          .error(function (data, status, headers, config) {
              if (data && data.error)
                  jlog('[error] ' + data.error);
          });
      }
      else  // remote: immediate get
      {
          $scope.onExportCSV(local);
      }
  }

  function csvExport(data) {

      $http.post($scope.csvUrl, data
      ).success(function (res) {
          loadData();
          dialog.error($translate.instant("_SUCCESS"), $translate.instant("_PASSWORD_IMPORT_SUCCESS", res));
      }).error(function (err) {
          jlog("CSV import error: " + err);
          $rootScope.loading = false;
          dialog.error($translate.instant("_STATE_ERROR"), $translate.instant("_PASSWORD_IMPORT_ERROR"));
      })
  }

  function loadData() {
      $rootScope.loading = true;
      $http.get(credentialsUrl).success(function (res) {
          $scope.credentials = res;
          $scope.credentialsSupported = true;
      });
  }

  $scope.$on('$destroy', function (event) {

    jlog('Destroying Chromium Controller');

    utils.deinit();
    poller.deinit();
  });

  loadData();

  poller.regPostCb(pollPostCb);
  poller.subscribe(['chromium']);
  pollPostCb();

  $rootScope.viewTitle = $filter('translate')('_BROWSER');

}]);
